
# q-expansion of newform 3568.2.a.f, downloaded from the LMFDB on 14 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3568
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [0], [0], [-1], [-2], [1], [4], [-1], [-3], [10], [-3], [-5], [6], [-6], [-9], [1], [4], [-9], [-4], [-5], [0], [-14], [-5], [2], [2], [-13], [-12], [-10], [6], [2], [10], [-2], [-22], [22], [9], [2], [8], [1], [-16], [4], [9], [3], [-4], [6], [14], [18], [1], [8], [8], [4], [6], [13], [-10], [-9], [-9], [-16], [-5], [-8], [-13], [6], [0], [-19], [-21], [-26], [21], [7], [14], [-16], [15], [-14], [36], [10], [16], [-4], [-1], [37], [8], [-26], [6], [-24], [10], [15], [14], [15], [24], [-30], [36], [17], [8], [-9], [-6], [4], [-15], [8], [-9], [-18], [-18], [40], [-10], [2], [-28], [20], [-22], [37], [-7], [-1], [-44], [40], [-22], [-33], [-32], [-29], [-21], [-31], [-2], [16], [-32], [-14], [-12], [24], [-1], [50], [28], [20], [15], [28], [16], [16], [13], [52], [-6], [-14], [-34], [24], [35], [16], [-3], [11], [-36], [-4], [21], [-4], [9], [-36], [-23], [28], [-14], [-36], [21], [52], [-39], [36], [6], [16], [8], [-48], [51], [54], [59], [-6], [36], [20], [-13], [-8], [0], [56], [10], [-38], [-47], [-36], [52], [-18], [38], [-33], [18], [-17], [-9], [-26], [-35], [47], [32], [35], [-22], [-24], [-6], [-1], [36], [-58], [-8], [-7], [-36], [12], [13], [38], [-66], [19], [-37], [0], [-48], [-34], [26], [-38], [-18], [32], [-61], [34], [7], [4], [63], [-48], [22], [30], [37], [-27], [-50], [50], [-8], [-58], [32], [-42], [-49], [-54], [-31], [54], [-28], [-24], [-24], [44], [-69], [-37], [-14], [22], [10], [-41], [-64], [20], [24], [41], [63], [-36], [29], [-32], [-61], [-12], [-59], [27], [17], [-52], [-11], [-37], [56], [-16], [44], [-54], [-38], [-7], [-18], [-43], [26], [31], [-40], [24], [-41], [-71], [12], [30], [66], [34], [29], [50], [29], [-28], [-33], [-48], [-47], [68], [-32], [13], [-48], [-66], [-68], [-36], [-8], [-54], [45], [-34], [-78], [53], [-47], [68], [-28], [0], [-73], [40], [52], [64], [56], [21], [10], [-42], [-67], [-5], [-56], [50], [22], [-68], [-56], [38], [-28], [-70], [-70], [14], [-44], [-25], [58], [0], [-55], [26], [-22], [10], [53], [20], [54], [9], [-8], [34], [-16], [26], [-6], [-38], [-63], [0], [4], [-55], [8], [26], [9], [-20], [-83], [-61], [18], [-6], [80], [54], [78], [-80], [1], [34], [42], [27], [-42], [54], [26], [-38], [-58], [18], [-22], [54], [4], [-92], [-58], [-10], [-15], [66], [88], [30], [-58], [-3], [-36], [8], [25], [-84], [24], [-12], [-48], [-64], [28], [78], [-82], [-40], [72], [64], [90], [44], [-40], [-4], [-67], [-54], [-88], [100], [-42], [-36], [42], [-15], [99], [34], [-75], [-88], [39], [5], [-72], [-27], [3], [-18], [68], [-15], [-6], [-34], [-20], [84], [-6], [89], [-20], [34], [104], [-66], [12], [96], [52], [30], [63], [-8], [-45], [62], [-78], [-48], [-56], [-38], [-36]]
